CREATE OR REPLACE package htmlform
------------------------------------------------------------------------
-- Project: Web Applications 101
--
-- Purpose: This package contains procedures to handle HTML forms
--
-- Author:  Sten Vesterli, sten@vesterli.com, www.vesterli.com
--
-- Version When       Who         What
-- ------- ---------- ----------- --------------------------------------
-- 2.1     2001-05-21 S.Vesterli  Bugfixes, more dynamic SQL features
-- 2.0     2001-01-15 S.Vesterli  Updated for Web Apps 101
-- 1.0     1998-09-14 S.Vesterli  Published in Oracle Developer
------------------------------------------------------------------------
as
  -- global name of form label CSS class
  formLabelClass varchar2(250) := 'formLabel';

  -- start and end HTML form
  function formStart(action in varchar2,
      border in number default 0) return varchar2;
  function formEnd return varchar2;

  -- text fields
  function textField(label in varchar2, name in varchar2,
      maxlength in number default null,
      fieldSize in number default null) return varchar2;
  function textField(label in varchar2, name in varchar2,
      value in varchar2, maxlength in number default null,
      fieldSize in number default null) return varchar2;
  function keyField(label in varchar2, name in varchar2,
      value in varchar2) return varchar2;
  function updateField(label in varchar2, name in varchar2,
      value in varchar2, maxlength in number default null,
      fieldSize in number default null) return varchar2;

  -- password field
  function passwordField (label in varchar2, name in varchar2,
      maxlength in varchar2 default null,
      fieldSize in varchar2 default null) return varchar2;

  -- hidden field
  function hiddenField (name in varchar2, value in varchar2)
      return varchar2;

  -- text area
  function textarea(label in varchar2, name in varchar2,
      value in varchar2 default null,
      rows in number default null, cols in number default null)
      return varchar2;

  -- select lists
  function selectStart(label in varchar2, name in varchar2)
      return varchar2;
  function selectStart(label in varchar2, name in varchar2,
      listSize in number, multiple in boolean default true)
      return varchar2;
  function selectOption(text in varchar2,
      value in varchar2 default null) return varchar2;
  function selectEnd return varchar2;
  function selectTable(label in varchar2, name in varchar2,
      tableName in varchar2, textCol in varchar2,
      valueCol in varchar2 default null,
      whereCondition in varchar2 default null,
      orderBy in varchar2 default null) return varchar2;
  function selectTable(label in varchar2, name in varchar2,
      tableName in varchar2, textCol in varchar2,
      listSize in number, multiple in boolean default true,
      valueCol in varchar2 default null,
      whereCondition in varchar2 default null,
      orderBy in varchar2 default null) return varchar2;
  function selectQuery(label in varchar2, name in varchar2,
      query in varchar2, listSize in number default null,
      multiple in boolean default true) return varchar2;

  -- checkbox
  function checkbox(label in varchar2, name in varchar2,
      value in varchar2 default null, checked in boolean default false)
      return varchar2;
  function checkbox(label in varchar2, name in varchar2,
      checked in boolean) return varchar2;

  -- radio buttons
  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      selectedValue in number default null) return varchar2;
  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      label3 in varchar2, value3 in varchar2,
      selectedValue in number default null) return varchar2;
  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      label3 in varchar2, value3 in varchar2,
      label4 in varchar2, value4 in varchar2,
      selectedValue in number default null) return varchar2;
  function radiobutton(name in varchar2,
      label_arr in owa.vc_arr, value_arr in owa.vc_arr,
      selectedValue in number default null) return varchar2;

  -- a row for buttons (or anything else)
  function buttonsStart return varchar2;
  function buttonsEnd return varchar2;

  -- create buttons
  function submitButton(label in varchar2,
      name in varchar2 default null) return varchar2;
  function button(label in varchar2, destination in varchar2)
      return varchar2;

  -- a row with just a submit button
  function submitButtonRow(label in varchar2) return varchar2;
end htmlform;
/

CREATE OR REPLACE package body htmlform
as
  ----------------------------------------------------------------------
  -- local modules
  ----------------------------------------------------------------------
  -- return different types of HTML input tags
  function fieldTag(name varchar2 default null, tagType varchar2,
      value in varchar2 default null, maxlength in number default null,
      fieldSize in number default null) return varchar2
  is
    sb varchar2(2000);
  begin
    sb :='      <input type="' || tagType || '"';
    if name is not null
    then
      sb := sb || ' name="' || name || '"';
    end if;
    if value is not null
    then
      sb := sb || ' value="' || value || '"';
    end if;
    if maxLength is not null
    then
      sb := sb || ' maxlength="' || maxLength || '"';
    end if;
    if fieldSize is not null
    then
      sb := sb || ' size="' || fieldSize || '"';
    end if;
    sb := sb || ' />';
    return sb;
  end fieldTag;

  -- return a table cell containing a field label
  function fieldLabelCell(label in varchar2)
      return varchar2
  is
    sb varchar2(2000);
  begin
    sb := '    <td class="' || formLabelClass || '">'
        || label || '&nbsp;&nbsp;</td>';
    return sb;
  end fieldLabelCell;

  -- return a table row with a label cell and a cell with an input tag
  function inputField(label varchar2, name varchar2,
      value in varchar2 default null, tagType varchar2,
      maxlength in varchar2 default null,
      fieldSize in varchar2 default null) return varchar2
  is
    sb varchar2(2000);
  begin
    sb :=
           '  <tr> ' || chr(10)
        || fieldLabelCell(label) || chr(10)
        || '    <td>' || chr(10)
        || fieldTag(name, tagType, value, maxlength, fieldSize)
        || chr(10)
        || '    </td>' || chr(10)
        || '  </tr>'
        ;
    return sb;
  end inputField;

  ----------------------------------------------------------------------
  -- global modules
  ----------------------------------------------------------------------
  -- start an HTML form and associated HTML table
  function formStart(action in varchar2,
      border in number default 0) return varchar2
  is
    s varchar2(2000);
  begin
    s := '<form method="post" action="'
        || action || '">' || chr(10)
        || '<table border="' || border || '">'
    ;
    return s;
  end;

  -- end HTML form and table
  function formEnd return varchar2
  is
  begin
    return '</table>' || chr(10) || '</form>';
  end;

  -- text fields
  function textField(label in varchar2, name in varchar2,
      maxlength in number default null,
      fieldSize in number default null) return varchar2
  is
  begin
    return inputField(label, name, null, 'text', maxlength, fieldSize);
  end;

  function textField(label in varchar2, name in varchar2,
      value in varchar2 , maxlength in number default null,
      fieldSize in number default null) return varchar2
  is
  begin
    return inputField(label, name, value, 'text', maxlength, fieldSize);
  end;

  function keyField(label in varchar2, name in varchar2,
      value in varchar2) return varchar2
  is
    sb varchar2(2000);
  begin
    sb := '  <tr> ' || chr(10)
    || fieldLabelCell(label) || chr(10)
    || '    <td>' || chr(10)
    || '      ' || value || chr(10)
    || hiddenField(name, value)
    || '    </td>' || chr(10)
    || '  </tr>'
    ;
    return sb;
  end;

  function updateField(label in varchar2, name in varchar2,
      value in varchar2, maxlength in number default null,
      fieldSize in number default null) return varchar2
  is
    sb varchar2(2000);
  begin
    sb :=
           '  <tr> ' || chr(10)
        || fieldLabelCell(label) || chr(10)
        || '    <td>' || chr(10)
        || fieldTag('new' || name, 'text', value, maxlength, fieldSize)
        || chr(10)
        || hiddenField('old' || name, value) || chr(10)
        || '    </td>' || chr(10)
        || '  </tr>'
        ;
    return sb;
  end;

  -- password field
  function passwordField (label in varchar2, name in varchar2,
      maxlength in varchar2 default null,
      fieldSize in varchar2 default null) return varchar2
  is
  begin
    return inputField(label, name, null, 'password', maxlength,
        fieldSize);
  end;

  -- hidden field
  function hiddenField (name in varchar2, value in varchar2)
      return varchar2
  is
  begin
    return '      '
        || '<input type="hidden" name="' || name
        || '" value="' || value || '" />';
  end;

  -- text area
  function textarea(label in varchar2, name in varchar2,
      value in varchar2 default null,
      rows in number default null, cols in number default null)
      return varchar2
  is
    sb varchar2(2000);
  begin
    sb :=  '  <tr>' || chr(10)
        || '    <td colspan="2" class="' || formLabelClass || '">'
        || label || '<br />' || chr(10)
        || '<textarea';
    if rows is not null
    then
      sb := sb || ' rows="' || rows || '"';
    end if;
    if cols is not null
    then
      sb := sb || ' cols="' || cols || '"';
    end if;
    sb := sb || '>';
    if value is not null
    then
      sb := sb || value;
    end if;
    sb := sb || '</textarea>' || chr(10) || '    </td>';
    return sb;
  end;

    -- select lists
  function selectStart(label in varchar2, name in varchar2)
      return varchar2
  is
    sb varchar2(2000);
  begin
    sb :=  '  <tr>' || chr(10)
        || fieldLabelCell(label) || chr(10)
        || '    <td>' || chr(10)
        || '      <select name="' || name || '">';
    return sb;
  end;

  function selectStart(label in varchar2, name in varchar2,
      listSize in number, multiple in boolean default true)
      return varchar2
  is
    sb varchar2(2000);
  begin
    sb :=  '  <tr>' || chr(10)
        || fieldLabelCell(label) || chr(10)
        || '    <td>' || chr(10)
        || '      <select name="' || name || '"';
    if listSize is not null
    then
      sb := sb || ' size="' || listSize || '"';
    end if;
    if multiple
    then
      sb := sb || ' multiple="multiple"';
    end if;
    sb := sb || '>';
    return sb;
  end;

  function selectOption(text in varchar2,
      value in varchar2 default null) return varchar2
  is
    sb varchar2(2000);
  begin
    sb := '        <option';
    if value is not null
    then
      sb := sb || ' value="' || value || '"';
    end if;
    sb := sb || '>' || text || '</option>';
    return sb;
  end;

  function selectEnd return varchar2
  is
  begin
    return '      </select>' || chr(10)
        || '    </td>' || chr(10)
        || '  </tr>';
  end;

  function selectTable(label in varchar2, name in varchar2,
      tableName in varchar2, textCol in varchar2,
      valueCol in varchar2 default null,
      whereCondition in varchar2 default null,
      orderBy in varchar2 default null) return varchar2
  is
  begin
    return selectTable(label, name, tablename, textCol, 1, false,
        valueCol, whereCondition, orderBy);
  end;

  function selectTable(label in varchar2, name in varchar2,
      tableName in varchar2, textCol in varchar2,
      listSize in number, multiple in boolean default true,
      valueCol in varchar2 default null,
      whereCondition in varchar2 default null,
      orderBy in varchar2 default null) return varchar2
  is
    lvQuery varchar2(2000);
  begin
    lvQuery := 'select ' || textcol;
    if valueCol is not null
    then
      lvQuery := lvQuery || ', ' || valueCol;
    else
      lvQuery := lvQuery || ', ' || textCol;
    end if;
    lvQuery := lvQuery || ' from ' || tableName;
    if whereCondition is not null
    then
      if upper(substr(whereCondition, 1, 6)) = 'WHERE '
      then
        lvQuery := lvQuery || ' ' || whereCondition;
      else
        lvQuery := lvQuery || ' where ' || whereCondition;
      end if;
    end if;
    if orderBy is not null
    then
      if upper(substr(orderBy, 1, 9)) = 'ORDER BY '
      then
        lvQuery := lvQuery || ' ' || orderBy;
      else
        lvQuery := lvQuery || ' order by ' || orderBy;
      end if;
    else
      lvQuery := lvQuery || ' order by ' || textCol;
    end if;
    debug.log(1, lvQuery);
    return selectQuery(label, name, lvQuery, listSize, multiple);
  end;

  function selectQuery(label in varchar2, name in varchar2,
      query in varchar2, listSize in number default null,
      multiple in boolean default true) return varchar2
  is
    sb varchar2(4000);
    liSqlCursor integer;
    lvValue varchar2(2000);
    lvText varchar2(2000);
    liExecFeedback integer;
  begin
    sb := selectStart(label, name, listSize, multiple) || chr(10);
    liSqlCursor := dbms_sql.open_cursor;
    if debug.doingSql
    then
      begin
        dbms_sql.parse(
          liSqlCursor
        , query
        , dbms_sql.native
        );
        dbms_sql.define_column (liSqlCursor, 1, lvValue, 2000);
        dbms_sql.define_column (liSqlCursor, 2, lvText, 2000);
        liExecFeedback := dbms_sql.execute(liSqlCursor);
        while dbms_sql.fetch_rows (liSqlCursor) != 0 loop
          dbms_sql.column_value (liSqlCursor, 1, lvValue);
          dbms_sql.column_value (liSqlCursor, 2, lvText);
          sb := sb || selectOption(lvValue, lvText) || chr(10);
        end loop;
        dbms_sql.close_cursor(liSqlCursor);
        exception
          when others
          then
            sb := sb || '<!-- FIXME: Error in SQL: '
                || sqlerrm || ' -->' || chr(10);
        end;
      end if;
    sb := sb || selectEnd;
    return sb;
  end;

  -- checkbox
  function checkbox(label in varchar2, name in varchar2,
      value in varchar2 default null, checked in boolean default false)
      return varchar2
  is
    sb varchar2(2000);
  begin
    sb := '  <tr>' || chr(10)
        || '    <td class="' || formLabelClass || '">'
        || label
        || '</td>' || chr(10)
        || '    <td><input type="checkbox" name="'
        || name
        || '"'
    ;
    if value is not null
    then
      sb := sb || ' value="' || value || '"';
    end if;
    if checked
    then
      sb := sb || ' checked="checked"';
    end if;
    sb := sb || '></td>' || chr(10) || '  </tr>';
    return sb;
  end;

  function checkbox(label in varchar2, name in varchar2,
      checked in boolean) return varchar2
  is
  begin
    return checkbox(label, name, null, checked);
  end;

  -- radio buttons
  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      selectedValue in number default null) return varchar2
  is
    ltLabels owa.vc_arr;
    ltValues owa.vc_arr;
  begin
    ltLabels(1) := label1;
    ltValues(1) := value1;
    ltLabels(2) := label2;
    ltValues(2) := value2;
    return radiobutton(name, ltLabels, ltValues, selectedValue);
  end;

  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      label3 in varchar2, value3 in varchar2,
      selectedValue in number default null) return varchar2
  is
    ltLabels owa.vc_arr;
    ltValues owa.vc_arr;
  begin
    ltLabels(1) := label1;
    ltValues(1) := value1;
    ltLabels(2) := label2;
    ltValues(2) := value2;
    ltLabels(3) := label3;
    ltValues(3) := value3;
    return radiobutton(name, ltLabels, ltValues, selectedValue);
  end;

  function radiobutton(name in varchar2,
      label1 in varchar2, value1 in varchar2,
      label2 in varchar2, value2 in varchar2,
      label3 in varchar2, value3 in varchar2,
      label4 in varchar2, value4 in varchar2,
      selectedValue in number default null) return varchar2
  is
    ltLabels owa.vc_arr;
    ltValues owa.vc_arr;
  begin
    ltLabels(1) := label1;
    ltValues(1) := value1;
    ltLabels(2) := label2;
    ltValues(2) := value2;
    ltLabels(3) := label3;
    ltValues(3) := value3;
    ltLabels(4) := label4;
    ltValues(4) := value4;
    return radiobutton(name, ltLabels, ltValues, selectedValue);
  end;

  function radiobutton(name in varchar2,
      label_arr in owa.vc_arr, value_arr in owa.vc_arr,
      selectedValue in number default null) return varchar2
  is
   sb varchar2(2000);
  begin
    sb :=  '  <tr>' || chr(10)
        || '    <td colspan="2" class="'
        || formLabelClass
        || '">' || chr(10);
    if label_arr.count != value_arr.count
    then
      sb := sb || '<!-- FIXME: Number of labels ('
          || label_arr.count || ') and number of values ('
          || value_arr.count || ') must match -->' || chr(10)
      ;
    else
      -- separate block so we can capture gaps in index values
      begin
        for i in 1..label_arr.count loop
          sb := sb || '      ' || label_arr(i)
              || '<input type="radio" name="'
              || name || '" value="' || value_arr(i)
              || '"'
          ;
          if i = selectedValue
          then
            sb := sb || ' checked="checked"';
          end if;
          sb := sb || '>' || chr(10);
        end loop;
      exception
        when no_data_found
        then
          -- there was a gap in the index values, can't handle that
          sb := sb || '<!-- FIXME: Could not create remaining '
              || 'radio buttons. Check index values for label and '
              || 'value arrays: They must be 1, 2, 3, .. '
              || 'starting from 1 and without gaps -->' || chr(10)
          ;
      end;
    end if;
    sb := sb || '    </td>' || chr(10) || '  </tr>';
    return sb;
  end;

  -- a row for buttons (or anything else)
  function buttonsStart return varchar2
  is
  begin
    return '  <tr>' || chr(10) || '    <td colspan="2">';
  end;

  function buttonsEnd return varchar2
  is
  begin
    return '    </td>' || chr(10) || '  </tr>';
  end;

  -- create buttons
  function submitButton(label in varchar2,
      name in varchar2 default null) return varchar2
  is
  begin
    return fieldTag(name, 'submit', label);
  end;

  function button(label in varchar2, destination in varchar2)
      return varchar2
  is
    s varchar2(2000);
  begin
    s := '      <input type="button" value="'
      || label
      || '"'
      || '	onClick="document.location.href='''
      || destination
      || '''" />' || chr(10)
    ;
    return s;
  end;

  -- a row with just a submit button
  function submitButtonRow(label in varchar2) return varchar2
  is
    sb varchar2(2000);
  begin
    sb := buttonsStart
      || '' || chr(10)
      || submitButton(label)
      || '' || chr(10)
      || buttonsEnd
    ;
    return sb;
  end;
begin
debug.doSql;
 debug.setLevel(0);
debug.SETDESTINATION(debug.destWeb);
end htmlform;
/

